import React, { FC, useContext, useMemo } from 'react';
import RcFieldForm, { FormInstance } from 'rc-field-form';
import { FormProps } from 'rc-field-form/es/Form';
import {
  Form as NativeBaseForm,
  View,
  Text,
  Row,
  Col,
  Grid,
} from 'native-base';
import CommonAdapters from './CommonAdapters';
import { styles } from './style';
import { ButtonColors } from '../Configurations/ThemeConfig';

export interface FormField {
  id: string;
  label?: NamePath;
  tips?: string;
  type: string | { object: FormSchema };
  props?: any;
  options?: any;
  view?: (value: any) => React.ReactNode;
}
export interface FormLayout {
  layout: 'horizontal' | 'vertical';
  fields: FormField[];
}
type NamePathArr = (string | number)[];
type NamePath = string | number | NamePathArr;

export interface FormFieldsProps {
  schema: FormSchema;
  layout?: 'horizontal' | 'vertical';
  adapters?: TypeAdapter[];
  validationAdapters?: ValidationAdapter[];
  className?: string;
  fieldData?: any;
  parentPath?: NamePathArr;
}

export interface TypeAdapter {
  name: string;
  render: (field: FormField, props: any) => React.ReactNode;
  initialValue?: (field: FormField, value?: any, item?: any) => any;
}

export interface ValidationAdapter {
  name: string;
  validator: any;
  message: string;
}

export type FormSchema = FormField | FormField[] | FormLayout | FormLayout[];
type SchemaFormItemProps = {
  field: FormField;
  adapters: { [key: string]: TypeAdapter };
  validationAdapters: { [key: string]: ValidationAdapter };
  fieldData?: any;
  _adapters?: TypeAdapter[];
  _validationAdapters?: ValidationAdapter[];
  parentPath: NamePathArr;
};
type FormXContextData = {
  value?: any;
  form: FormInstance;
};
const FormXContext = React.createContext<FormXContextData>({
  form: undefined as any,
});
const SchemaFormItem = (props: SchemaFormItemProps) => {
  const { field, adapters } = props;
  const fType = useMemo(() => {
    const fType = field.type;
    if (typeof fType === 'string') {
      return fType;
    }
    if (fType.hasOwnProperty('object')) {
      return 'object';
    }
    return 'array';
  }, [field.type]);
  const namePath: NamePath = useMemo(() => {
    const fName = props.fieldData?.name;
    if (fName === undefined) {
      return [field.id];
    }
    return [fName as any, field.id];
  }, [field.id, props]);
  const options = useMemo(() => {
    if (
      !field.options?.rules ||
      field.options!.rules.findIndex(
        (v: any) =>
          v.hasOwnProperty('validator') && typeof v.validator === 'string'
      ) === -1
    ) {
      return field.options;
    }
    const opts = { ...field.options };
    opts.rules = opts.rules.map((rule: any) => {
      if (
        !rule.hasOwnProperty('validator') ||
        typeof rule.validator !== 'string'
      ) {
        return rule;
      }

      const adapt = props.validationAdapters[rule.validator];
      if (adapt === undefined) {
        return rule;
      }
      const message = rule.message ? rule.message : adapt.message;
      return (form: FormInstance) => ({
        ...rule,
        message,
        validator: async (r: any, value: any) =>
          adapt.validator(r, value, form),
      });
    });
    return opts;
  }, [field.options, props.validationAdapters]);
  const form = useContext(FormXContext).form;
  const adapter = useMemo(
    () => (adapters[fType] ? adapters[fType] : adapters.string),
    [adapters, fType]
  );
  // const toSentenceCase = (val: string) => {
  //   const temp = val.replace(/([A-Z])/g, ' $1').replace(/[']/g, '');
  //   return temp.charAt(0).toUpperCase() + temp.slice(1);
  // };
  const repaceNamePath = (val: string) => {
    return val.replace(/'(.*?)'/g, '*');
  };
  return (
    <RcFieldForm.Field
      {...options}
      label={field.label}
      name={namePath}
      validateTrigger={'onFinish'}
    >
      {(ps: any) => (
        <View
          style={{
            flex: 1,
          }}
        >
          {adapter.render(field, {
            ...field.props,
            ...ps,
            ...field,
            validationInfo: form
              .getFieldError(namePath)
              .map((item) => repaceNamePath(item)),
            type: field?.props?.type,
          })}
          {/*{field?.label === undefined &&*/}
          {/*  form.getFieldError(namePath).length > 0 && (*/}
          <Text style={{ ...styles.errorText, color: field.props?.type !== 'material' ? ButtonColors.primary : '#5F4591' }}>
            {form.getFieldError(namePath).map((item) => repaceNamePath(item))}
          </Text>
          {/*)*/}
          {/*}*/}
        </View>
      )}
    </RcFieldForm.Field>
  );
};

type SchemaFormFieldItemType = FormField | FormField[];
type SchemaFormItemGroupProps = {
  field: SchemaFormFieldItemType;
  adapters: { [key: string]: TypeAdapter };
  validationAdapters: { [key: string]: ValidationAdapter };
  fieldData?: any;
  _adapters?: TypeAdapter[];
  _validationAdapters?: ValidationAdapter[];
  parentPath: NamePathArr;
};
const SchemaFormItemGroup = (props: SchemaFormItemGroupProps) => {
  const { field, adapters } = props;
  // console.log('from group ', props);
  return (
    <SchemaFormItem
      field={field as FormField}
      adapters={adapters}
      fieldData={props.fieldData}
      validationAdapters={props.validationAdapters}
      _adapters={props._adapters}
      _validationAdapters={props._validationAdapters}
      parentPath={props.parentPath}
    />
  );
};
type SchemaFormFieldsProps = {
  schema: FormField[];
  adapters: { [key: string]: TypeAdapter };
  validationAdapters: { [key: string]: ValidationAdapter };
  fieldData?: any;
  _adapters?: TypeAdapter[];
  _validationAdapters?: ValidationAdapter[];
  parentPath: NamePathArr;
};

const SchemaFormFields: FC<SchemaFormFieldsProps> = (
  props: SchemaFormFieldsProps
) => {
  const { schema, adapters, validationAdapters } = props;
  const form = useContext(FormXContext);
  const fields = useMemo(() => {
    const f: SchemaFormFieldItemType[] = [];
    let group: FormField[] = [];
    for (const field of schema) {
      if (group.length > 0) {
        f.push(group);
        group = [];
      }
      f.push(field);
    }
    if (group.length > 0) {
      f.push(group);
    }
    return f;
  }, [form.form, form.value, schema]);

  return (
    <>
      {fields.map((field, index) => (
        <SchemaFormItemGroup
          field={field}
          adapters={adapters}
          key={index}
          fieldData={props.fieldData}
          validationAdapters={validationAdapters}
          _adapters={props._adapters}
          _validationAdapters={props._validationAdapters}
          parentPath={props.parentPath}
        />
      ))}
    </>
  );
};

const SchemaFormItems: FC<FormFieldsProps> = (props: FormFieldsProps) => {
  const { schema } = props;
  const adapters: { [key: string]: TypeAdapter } = useMemo(() => {
    const ads: { [key: string]: TypeAdapter } = {};
    CommonAdapters.all().forEach((value) => (ads[value.name] = value));
    if (props.adapters) {
      props.adapters.forEach((value) => (ads[value.name] = value));
    }
    return ads;
  }, [props.adapters]);

  const validationAdapters: {
    [key: string]: ValidationAdapter;
  } = useMemo(() => {
    const ads: { [key: string]: ValidationAdapter } = {};
    CommonAdapters.allValidators().forEach(
      (value) => (ads[value.name] = value)
    );
    if (props.validationAdapters) {
      props.validationAdapters.forEach((value) => (ads[value.name] = value));
    }
    return ads;
  }, [props.validationAdapters]);
  const [sch, typ] = useMemo(() => {
    const sx = Array.isArray(schema) ? schema : [schema];
    const ty =
      sx.length > 0 && sx[0].hasOwnProperty('fields') ? 'columns' : 'fields';
    return [sx, ty];
  }, []);
  if (sch.length === 0) {
    return <></>;
  }
  if (typ === 'fields') {
    return (
      <SchemaFormFields
        schema={sch as FormField[]}
        adapters={adapters}
        validationAdapters={validationAdapters}
        fieldData={props.fieldData}
        _adapters={props.adapters}
        _validationAdapters={props.validationAdapters}
        parentPath={props.parentPath ? props.parentPath : []}
      />
    );
  }
  return (
    <Grid style={{ flexDirection: 'column' }}>
      {(sch as FormLayout[]).map((value: FormLayout, index) => {
        if (value.layout === 'vertical') {
          return (
            <Col key={index} size={24} style={{ flex: 1, width: '100%' }}>
              <SchemaFormFields
                schema={value.fields}
                adapters={adapters}
                validationAdapters={validationAdapters}
                fieldData={props.fieldData}
                parentPath={props.parentPath ? props.parentPath : []}
              />
            </Col>
          );
        }
        return (
          <Row {...value.layout} key={index} style={{ flex: 1, width: '100%' }}>
            <SchemaFormFields
              schema={value.fields}
              adapters={adapters}
              validationAdapters={validationAdapters}
              fieldData={props.fieldData}
              parentPath={props.parentPath ? props.parentPath : []}
            />
          </Row>
        );
      })}
    </Grid>
  );
};

interface FormEState {
  value?: any;
}

export default class Form extends React.Component<FormProps, FormEState> {
  state: FormEState = {};
  static Item = RcFieldForm.Field;
  static useForm = RcFieldForm.useForm;
  // static List = RcFieldForm.List;
  static Provider = RcFieldForm.FormProvider;
  static Items = SchemaFormItems;

  onValuesChange = (changed: any, all: any) => {
    this.setState({ value: all });
    if (this.props.onValuesChange) {
      this.props.onValuesChange(changed, all);
    }
  };

  render(): React.ReactNode {
    return (
      <FormXContext.Provider
        value={{ value: this.state.value, form: this.props.form! }}
      >
        <RcFieldForm
          {...this.props}
          onValuesChange={this.onValuesChange}
          component={NativeBaseForm}
        >
          {this.props.children}
        </RcFieldForm>
      </FormXContext.Provider>
    );
  }
}

export type FormComponentProps = { form: FormInstance };
