import React, { FC, useContext, useEffect, useState } from 'react';
import { Item, Label, NativeBase, Text, View } from 'native-base';
import { ScrollView, TouchableOpacity } from 'react-native';
import Icon from 'react-native-vector-icons/AntDesign';
import { styles } from './style';
import { ItemProp } from '../MultiSelect/MultiSelectDropDown';
import { DropdownList } from './DropdownList';
import TreeDropdownContext from './TreeDropdownContext';
import {
  borderedInputVariables,
  ButtonColors,
  primaryInputVariables,
} from '../../Configurations/ThemeConfig';

export type TreeItemProp = {
  parent: ItemProp;
  children: ItemProp[];
  color?: string;
};
export type RemoveItemProps = {
  parentId: string;
  childId: string;
  action: 'remove' | 'neutral';
};
export type TreeDropdownProps = {
  type: 'bordered' | 'primary';
  pickerList: TreeItemProp[];
  parentName: string;
  childName: string;
  label?: string;
  labelProps?: NativeBase.Label;
  onChange: (val?: string[]) => void;
  validationInfo?: string[];
  value?: any[];
  placeholder?: string;
  shadow?: boolean;
};
export const TreeDropdown: FC<TreeDropdownProps> = (
  props: TreeDropdownProps
) => {
  const {
    label,
    labelProps,
    pickerList,
    onChange,
    value,
    type,
    childName,
    parentName,
    placeholder,
    shadow,
  } = props;
  const [showPickerList, setShowPickerList] = useState<boolean>(false);
  const context = useContext(TreeDropdownContext).current!;
  const { removeTag, setSelectedList, selectedList } = context;
  useEffect(() => {
    if (value) {
      setSelectedList(value);
    }
  }, [value]);
  const renderTags = (placeholderColor?: string) => {
    if (selectedList.length === 0) {
      return (
        <Text
          onPress={() => setShowPickerList(!showPickerList)}
          style={{
            marginTop: 5,
            marginBottom: 5,
            color: placeholderColor ?? '#C4C4C4',
          }}
        >
          {placeholder ?? 'Click to select'}
        </Text>
      );
    }
    return selectedList.map((item: any, ind: number) => {
      const originalVal = pickerList.find(
        (clr) => clr.parent.value === item[parentName]
      );
      return item[childName].map((val: string, index: number) => (
        <View
          style={[
            styles.tagContainer,
            {
              backgroundColor: originalVal?.color ?? ButtonColors.primary,
            },
          ]}
          key={`${ind}${index}`}
        >
          <Text
            numberOfLines={1}
            onPress={() => setShowPickerList(!showPickerList)}
            style={{ color: 'white' }}
          >
            {originalVal?.parent.label} {` `}{' '}
            {
              originalVal?.children.find((x: ItemProp) => x.value === val)
                ?.label
            }
          </Text>
          <TouchableOpacity
            style={{ paddingLeft: 20, paddingRight: 20 }}
            onPress={() =>
              removeTag({
                action: 'remove',
                childId: val,
                parentId: item[parentName],
              })
            }
          >
            <Icon
              style={{ color: 'white', fontSize: 15 }}
              name={'closecircleo'}
            />
          </TouchableOpacity>
        </View>
      ));
    });
  };
  const onComplete = (val: any[]) => {
    onChange(val.length === 0 ? undefined : val);
    setSelectedList(val);
  };
  const renderElement = (
    selectStyleX?: any,
    labelStyle?: any,
    placeholderColor?: string
  ) => {
    selectStyleX = shadow
      ? { ...selectStyleX, ...styles.shadow }
      : selectStyleX;
    return (
      <Item stackedLabel={label !== undefined} style={styles.itemStyle}>
        {label && (
          <Label
            {...labelProps}
            style={
              labelProps?.style
                ? { ...labelStyle, ...(labelProps?.style as object) }
                : labelStyle
            }
          >
            {label}
          </Label>
        )}
        <TouchableOpacity
          style={styles.default}
          onPress={() => setShowPickerList(!showPickerList)}
        >
          {selectedList.length === 0 ? (
            <View style={selectStyleX}>
              <TouchableOpacity style={[{ flexDirection: 'row' }]}>
                {renderTags(placeholderColor)}
              </TouchableOpacity>
            </View>
          ) : (
            <View onStartShouldSetResponder={() => true} style={selectStyleX}>
              <ScrollView
                horizontal={true}
                style={{ flexDirection: 'row', flex: 1 }}
              >
                <TouchableOpacity
                  style={[{ flexDirection: 'row' }]}
                  onPress={() => setShowPickerList(!showPickerList)}
                >
                  {renderTags()}
                </TouchableOpacity>
                <Text
                  style={[
                    {
                      alignSelf: 'stretch',
                      color: 'transparent',
                    },
                  ]}
                  onPress={() => setShowPickerList(!showPickerList)}
                >
                  {' '}
                  ************* **********
                </Text>
              </ScrollView>
            </View>
          )}
          <DropdownList
            onChange={onComplete}
            parentName={parentName}
            childName={childName}
            data={pickerList}
            showModal={showPickerList}
            closeModal={() => setShowPickerList(false)}
            value={value}
          />
        </TouchableOpacity>
      </Item>
    );
  };
  switch (type) {
    case 'bordered':
      return renderElement(
        { ...styles.bordered },
        {
          ...styles.textDefault,
          color: borderedInputVariables.labelColor,
          marginLeft: 10,
          fontWeight: 'bold',
        },
        borderedInputVariables.placeholderColor
      );
    default:
      return renderElement(
        styles.multiSelect,
        styles.textDefault,
        primaryInputVariables.placeholderColor
      );
  }
};
